@@ -444,6 +444,7 @@ const char *pqxx::connection_base::ErrMsg() const
 
 pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 {
+  // TODO: "Single-query" vsn based on PQexecParams() & private multi-query one
   activate();
 
   result R( PQexec(m_Conn, Query) );
@@ -463,6 +464,30 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 }
 
 
+pqxx::result pqxx::connection_base::exec_prepared(const char QueryName[],
+	int NumParams,
+	const char *const *Params,
+	int Retries)
+{
+  activate();
+  result R(PQexecPrepared(m_Conn, QueryName, NumParams, Params, NULL, NULL, 0));
+
+  while ((Retries > 0) && !R && !is_open())
+  {
+    Retries--;
+    Reset();
+    if (is_open()) 
+      R = PQexecPrepared(m_Conn, QueryName, NumParams, Params, NULL, NULL, 0);
+  }
+
+  // TODO: Is this really right?  What if there's just no memory?
+  if (!R) throw broken_connection();
+  R.CheckStatus(QueryName);
+  get_notifs();
+  return R;
+}
+
+
 void pqxx::connection_base::Reset()
 {
   clear_fdmask();
@@ -584,7 +609,7 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
   bool Result;
 
 #ifdef PQXX_HAVE_PQPUTCOPY
-  char *Buf;
+  char *Buf = 0;
   switch (PQgetCopyData(m_Conn, &Buf, false))
   {
     case -2:
@@ -636,7 +661,12 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
 
   Result = (Line != theWriteTerminator);
 
-  if (!Result) 
+  if (Result) 
+  {
+    if (!Line.empty() && (Line[Line.size()-1] == '\n'))
+      Line.erase(Line.size()-1);
+  }
+  else
   {
     Line.erase();
     if (PQendcopy(m_Conn)) throw runtime_error(ErrMsg());
@@ -647,7 +677,6 @@ bool pqxx::connection_base::ReadCopyLine(string &Line)
 }
 
 
-#include<iostream>// DEBUG CODE
 bool pqxx::connection_base::WriteCopyLine(const string &Line, bool async)
 {
   if (!is_open())
@@ -687,26 +716,26 @@ void pqxx::connection_base::EndCopyWrite()
 {
 #ifdef PQXX_HAVE_PQPUTCOPY
   go_sync();
-  int Res;
-  do
+  int Res = PQputCopyEnd(m_Conn, NULL);
+  switch (Res)
   {
-    Res = PQputCopyEnd(m_Conn, NULL);
-    switch (Res)
-    {
-      case -1:
-        throw runtime_error("Write to table failed: " + string(ErrMsg()));
-      case 0:
-        throw logic_error("libpqxx internal error: "
+  case -1:
+    throw runtime_error("Write to table failed: " + string(ErrMsg()));
+  case 0:
+    throw logic_error("libpqxx internal error: "
 	    "table write is inexplicably asynchronous");
-      case 1:
-        for (result R(PQgetResult(m_Conn)); R; R=PQgetResult(m_Conn))
-	    R.CheckStatus("[END COPY]");
-        break;
-      default:
-        throw logic_error("libpqxx internal error: "
+  case 1:
+    // Normal termination.  Retrieve result object.
+    break;
+
+  default:
+    throw logic_error("libpqxx internal error: "
 	    "unexpected result " + to_string(Res) + " from PQputCopyEnd()");
-    }
-  } while (!Res);
+  }
+
+  const result R(PQgetResult(m_Conn));
+  R.CheckStatus("[END COPY]");
+
 #else
   WriteCopyLine(theWriteTerminator);
   // This check is a little odd, but for some reason PostgreSQL 7.4 keeps
